"""
Global Constant Definitions
"""

from qtpy.QtGui import QColor


MIN_REDRAW_RATE_HZ = 1
MAX_REDRAW_RATE_HZ = 240
DEFAULT_REDRAW_RATE_HZ = 30

MIN_DATA_SAMPLING_RATE_HZ = 1
MAX_DATA_SAMPLING_RATE_HZ = 360
DEFAULT_DATA_SAMPLING_RATE_HZ = 10

DEFAULT_EXPORTED_IMAGE_WIDTH = "800"
DEFAULT_EXPORTED_IMAGE_HEIGHT = "600"

DEFAULT_CHART_BACKGROUND_COLOR = QColor("black")
DEFAULT_CHART_AXIS_COLOR = QColor("white")
DEFAULT_CHART_TITLE = "TimeChart"

MAX_DISPLAY_PV_NAME_LENGTH = 30

ASYNC_DATA_SAMPLING = 0
SYNC_DATA_SAMPLING = 1

X_AXIS_LABEL_SEPARATOR = " -- "
IMPORT_FILE_FORMAT = "TimeChart or EPICS StripTool file (*.json *.stp)"

